@@ -39,23 +39,22 @@
 #include "private.h"
 #include "mls.h"
 
-int mls_to_string(
-	sepol_handle_t* handle,
-	const policydb_t* policydb, 
-	const context_struct_t* mls, 
-	char** str) {
+int mls_to_string(sepol_handle_t * handle,
+		  const policydb_t * policydb,
+		  const context_struct_t * mls, char **str)
+{
 
 	char *ptr = NULL, *ptr2 = NULL;
 
 	/* Temporary buffer - length + NULL terminator */
 	int len = mls_compute_context_len(policydb, mls) + 1;
 
-	ptr = (char*) malloc(len);
+	ptr = (char *)malloc(len);
 	if (ptr == NULL)
 		goto omem;
 
 	/* Final string w/ ':' cut off */
-	ptr2 = (char*) malloc(len - 1);
+	ptr2 = (char *)malloc(len - 1);
 	if (ptr2 == NULL)
 		goto omem;
 
@@ -67,7 +66,7 @@ int mls_to_string(
 	*str = ptr2;
 	return STATUS_SUCCESS;
 
-	omem:
+      omem:
 	ERR(handle, "out of memory, could not convert mls context to string");
 
 	free(ptr);
@@ -76,15 +75,14 @@ int mls_to_string(
 
 }
 
-int mls_from_string(
-	sepol_handle_t* handle,
-	const policydb_t* policydb, 
-	const char* str, 
-	context_struct_t* mls) {
+int mls_from_string(sepol_handle_t * handle,
+		    const policydb_t * policydb,
+		    const char *str, context_struct_t * mls)
+{
 
-	char* tmp = strdup(str);
-	char* tmp_cp = tmp;
-	if (!tmp) 
+	char *tmp = strdup(str);
+	char *tmp_cp = tmp;
+	if (!tmp)
 		goto omem;
 
 	if (mls_context_to_sid(policydb, '$', &tmp_cp, mls) < 0) {
@@ -92,14 +90,14 @@ int mls_from_string(
 		free(tmp);
 		goto err;
 	}
-	
+
 	free(tmp);
 	return STATUS_SUCCESS;
 
-	omem:
+      omem:
 	ERR(handle, "out of memory");
 
-	err:
+      err:
 	ERR(handle, "could not construct mls context structure");
 	return STATUS_ERR;
 }
@@ -108,9 +106,9 @@ int mls_from_string(
  * Return the length in bytes for the MLS fields of the
  * security context string representation of `context'.
  */
-int mls_compute_context_len(
-	const policydb_t *policydb, 
-	const context_struct_t * context) {
+int mls_compute_context_len(const policydb_t * policydb,
+			    const context_struct_t * context)
+{
 
 	unsigned int i, l, len, range;
 	ebitmap_node_t *cnode;
@@ -118,23 +116,30 @@ int mls_compute_context_len(
 	if (!policydb->mls)
 		return 0;
 
-	len = 1; /* for the beginning ":" */
+	len = 1;		/* for the beginning ":" */
 	for (l = 0; l < 2; l++) {
 		range = 0;
-		len += strlen(policydb->p_sens_val_to_name[context->range.level[l].sens - 1]);
+		len +=
+		    strlen(policydb->
+			   p_sens_val_to_name[context->range.level[l].sens -
+					      1]);
 
 		ebitmap_for_each_bit(&context->range.level[l].cat, cnode, i) {
-		  if (ebitmap_node_get_bit(cnode, i)) {
+			if (ebitmap_node_get_bit(cnode, i)) {
 				if (range) {
 					range++;
 					continue;
 				}
 
-				len += strlen(policydb->p_cat_val_to_name[i]) + 1;
+				len +=
+				    strlen(policydb->p_cat_val_to_name[i]) + 1;
 				range++;
 			} else {
 				if (range > 1)
-					len += strlen(policydb->p_cat_val_to_name[i - 1]) + 1;
+					len +=
+					    strlen(policydb->
+						   p_cat_val_to_name[i - 1]) +
+					    1;
 				range = 0;
 			}
 		}
@@ -144,7 +149,7 @@ int mls_compute_context_len(
 
 		if (l == 0) {
 			if (mls_level_eq(&context->range.level[0],
-			                 &context->range.level[1]))
+					 &context->range.level[1]))
 				break;
 			else
 				len++;
@@ -154,16 +159,14 @@ int mls_compute_context_len(
 	return len;
 }
 
-
 /*
  * Write the security context string representation of
  * the MLS fields of `context' into the string `*scontext'.
  * Update `*scontext' to point to the end of the MLS fields.
  */
-void mls_sid_to_context(
-	const policydb_t *policydb,
-	const context_struct_t * context,
-	char **scontext) {
+void mls_sid_to_context(const policydb_t * policydb,
+			const context_struct_t * context, char **scontext)
+{
 
 	char *scontextp;
 	unsigned int i, l, range, wrote_sep;
@@ -181,11 +184,15 @@ void mls_sid_to_context(
 		range = 0;
 		wrote_sep = 0;
 		strcpy(scontextp,
-		       policydb->p_sens_val_to_name[context->range.level[l].sens - 1]);
-		scontextp += strlen(policydb->p_sens_val_to_name[context->range.level[l].sens - 1]);
+		       policydb->p_sens_val_to_name[context->range.level[l].
+						    sens - 1]);
+		scontextp +=
+		    strlen(policydb->
+			   p_sens_val_to_name[context->range.level[l].sens -
+					      1]);
 		/* categories */
 		ebitmap_for_each_bit(&context->range.level[l].cat, cnode, i) {
-		  if (ebitmap_node_get_bit(cnode, i)) {
+			if (ebitmap_node_get_bit(cnode, i)) {
 				if (range) {
 					range++;
 					continue;
@@ -196,8 +203,10 @@ void mls_sid_to_context(
 					wrote_sep = 1;
 				} else
 					*scontextp++ = ',';
-				strcpy(scontextp, policydb->p_cat_val_to_name[i]);
-				scontextp += strlen(policydb->p_cat_val_to_name[i]);
+				strcpy(scontextp,
+				       policydb->p_cat_val_to_name[i]);
+				scontextp +=
+				    strlen(policydb->p_cat_val_to_name[i]);
 				range++;
 			} else {
 				if (range > 1) {
@@ -206,8 +215,12 @@ void mls_sid_to_context(
 					else
 						*scontextp++ = ',';
 
-					strcpy(scontextp, policydb->p_cat_val_to_name[i - 1]);
-					scontextp += strlen(policydb->p_cat_val_to_name[i - 1]);
+					strcpy(scontextp,
+					       policydb->p_cat_val_to_name[i -
+									   1]);
+					scontextp +=
+					    strlen(policydb->
+						   p_cat_val_to_name[i - 1]);
 				}
 				range = 0;
 			}
@@ -225,7 +238,7 @@ void mls_sid_to_context(
 
 		if (l == 0) {
 			if (mls_level_eq(&context->range.level[0],
-			                 &context->range.level[1]))
+					 &context->range.level[1]))
 				break;
 			else {
 				*scontextp = '-';
@@ -242,14 +255,14 @@ void mls_sid_to_context(
  * Return 1 if the MLS fields in the security context
  * structure `c' are valid.  Return 0 otherwise.
  */
-int mls_context_isvalid(
-	const policydb_t *p, 
-	const context_struct_t * c) {
+int mls_context_isvalid(const policydb_t * p, const context_struct_t * c)
+{
 
 	level_datum_t *levdatum;
 	user_datum_t *usrdatum;
 	unsigned int i, l;
 	ebitmap_node_t *cnode;
+	hashtab_key_t key;
 
 	if (!p->mls)
 		return 1;
@@ -264,15 +277,20 @@ int mls_context_isvalid(
 		return 0;
 
 	for (l = 0; l < 2; l++) {
-		if (!c->range.level[l].sens || c->range.level[l].sens > p->p_levels.nprim)
+		if (!c->range.level[l].sens
+		    || c->range.level[l].sens > p->p_levels.nprim)
 			return 0;
-		levdatum = (level_datum_t *)hashtab_search(p->p_levels.table,
-			p->p_sens_val_to_name[c->range.level[l].sens - 1]);
+
+		key = p->p_sens_val_to_name[c->range.level[l].sens - 1];
+		if (!key)
+			return 0;
+
+		levdatum = (level_datum_t *) hashtab_search(p->p_levels.table, key);
 		if (!levdatum)
 			return 0;
 
 		ebitmap_for_each_bit(&c->range.level[l].cat, cnode, i) {
-		  if (ebitmap_node_get_bit(cnode, i)) {
+			if (ebitmap_node_get_bit(cnode, i)) {
 				if (i > p->p_cats.nprim)
 					return 0;
 				if (!ebitmap_get_bit(&levdatum->level->cat, i))
@@ -294,8 +312,8 @@ int mls_context_isvalid(
 	if (!c->user || c->user > p->p_users.nprim)
 		return 0;
 	usrdatum = p->user_val_to_struct[c->user - 1];
-	if (!mls_range_contains(usrdatum->range, c->range))
-		return 0; /* user may not be associated with range */
+	if (!usrdatum || !mls_range_contains(usrdatum->exp_range, c->range))
+		return 0;	/* user may not be associated with range */
 
 	return 1;
 }
@@ -310,11 +328,9 @@ int mls_context_isvalid(
  * This function modifies the string in place, inserting
  * NULL characters to terminate the MLS fields.
  */
-int mls_context_to_sid(
-	const policydb_t *policydb,
-	char oldc,
-	char **scontext,
-	context_struct_t * context) {
+int mls_context_to_sid(const policydb_t * policydb,
+		       char oldc, char **scontext, context_struct_t * context)
+{
 
 	char delim;
 	char *scontextp, *p, *rngptr;
@@ -339,10 +355,11 @@ int mls_context_to_sid(
 		*p++ = 0;
 
 	for (l = 0; l < 2; l++) {
-		levdatum = (level_datum_t *)hashtab_search(policydb->p_levels.table,
-					      (hashtab_key_t)scontextp);
+		levdatum =
+		    (level_datum_t *) hashtab_search(policydb->p_levels.table,
+						     (hashtab_key_t) scontextp);
 
-		if (!levdatum)	
+		if (!levdatum)
 			goto err;
 
 		context->range.level[l].sens = levdatum->level->sens;
@@ -363,13 +380,17 @@ int mls_context_to_sid(
 					*rngptr++ = 0;
 				}
 
-				catdatum = (cat_datum_t *)hashtab_search(policydb->p_cats.table,
-					      (hashtab_key_t)scontextp);
+				catdatum =
+				    (cat_datum_t *) hashtab_search(policydb->
+								   p_cats.table,
+								   (hashtab_key_t)
+								   scontextp);
 				if (!catdatum)
 					goto err;
 
-				if (ebitmap_set_bit(&context->range.level[l].cat,
-					catdatum->value - 1, 1))
+				if (ebitmap_set_bit
+				    (&context->range.level[l].cat,
+				     catdatum->s.value - 1, 1))
 					goto err;
 
 				/* If range, set all categories in range */
@@ -377,16 +398,22 @@ int mls_context_to_sid(
 					unsigned int i;
 
 					rngdatum = (cat_datum_t *)
-						hashtab_search(policydb->p_cats.table, 
-							(hashtab_key_t)rngptr);
+					    hashtab_search(policydb->p_cats.
+							   table,
+							   (hashtab_key_t)
+							   rngptr);
 					if (!rngdatum)
 						goto err;
 
-					if (catdatum->value >= rngdatum->value)
+					if (catdatum->s.value >=
+					    rngdatum->s.value)
 						goto err;
 
-					for (i = catdatum->value; i < rngdatum->value; i++) {
-						if (ebitmap_set_bit(&context->range.level[l].cat, i, 1))
+					for (i = catdatum->s.value;
+					     i < rngdatum->s.value; i++) {
+						if (ebitmap_set_bit
+						    (&context->range.level[l].
+						     cat, i, 1))
 							goto err;
 					}
 				}
@@ -410,15 +437,15 @@ int mls_context_to_sid(
 
 	/* High level is missing, copy low level */
 	if (l == 0) {
-		if (mls_level_cpy(&context->range.level[1], 
-			&context->range.level[0]) < 0)
+		if (mls_level_cpy(&context->range.level[1],
+				  &context->range.level[0]) < 0)
 			goto err;
 	}
 	*scontext = ++p;
 
 	return STATUS_SUCCESS;
 
-	err:
+      err:
 	return STATUS_ERR;
 }
 
@@ -445,8 +472,8 @@ static inline int mls_copy_context(context_struct_t * dst,
 /*
  * Copies the effective MLS range from `src' into `dst'.
  */
-static inline int mls_scopy_context(context_struct_t *dst,
-				    context_struct_t *src)
+static inline int mls_scopy_context(context_struct_t * dst,
+				    context_struct_t * src)
 {
 	int l, rc = 0;
 
@@ -465,7 +492,7 @@ static inline int mls_scopy_context(context_struct_t *dst,
 /*
  * Copies the MLS range `range' into `context'.
  */
-static inline int mls_range_set(context_struct_t *context, mls_range_t *range)
+static inline int mls_range_set(context_struct_t * context, mls_range_t * range)
 {
 	int l, rc = 0;
 
@@ -481,15 +508,15 @@ static inline int mls_range_set(context_struct_t *context, mls_range_t *range)
 	return rc;
 }
 
-int mls_setup_user_range(context_struct_t *fromcon, user_datum_t *user,
-                         context_struct_t *usercon, int mls)
+int mls_setup_user_range(context_struct_t * fromcon, user_datum_t * user,
+			 context_struct_t * usercon, int mls)
 {
 	if (mls) {
 		mls_level_t *fromcon_sen = &(fromcon->range.level[0]);
 		mls_level_t *fromcon_clr = &(fromcon->range.level[1]);
-		mls_level_t *user_low = &(user->range.level[0]);
-		mls_level_t *user_clr = &(user->range.level[1]);
-		mls_level_t *user_def = &(user->dfltlevel);
+		mls_level_t *user_low = &(user->exp_range.level[0]);
+		mls_level_t *user_clr = &(user->exp_range.level[1]);
+		mls_level_t *user_def = &(user->exp_dfltlevel);
 		mls_level_t *usercon_sen = &(usercon->range.level[0]);
 		mls_level_t *usercon_clr = &(usercon->range.level[1]);
 
@@ -525,8 +552,7 @@ int mls_setup_user_range(context_struct_t *fromcon, user_datum_t *user,
  * policy `oldp' to the values specified in the policy `newp'.
  */
 int mls_convert_context(policydb_t * oldp,
-			policydb_t * newp,
-			context_struct_t * c)
+			policydb_t * newp, context_struct_t * c)
 {
 	level_datum_t *levdatum;
 	cat_datum_t *catdatum;
@@ -538,9 +564,15 @@ int mls_convert_context(policydb_t * oldp,
 		return 0;
 
 	for (l = 0; l < 2; l++) {
-		levdatum = (level_datum_t *) hashtab_search(
-						    newp->p_levels.table,
-		   oldp->p_sens_val_to_name[c->range.level[l].sens - 1]);
+		levdatum =
+		    (level_datum_t *) hashtab_search(newp->p_levels.table,
+						     oldp->
+						     p_sens_val_to_name[c->
+									range.
+									level
+									[l].
+									sens -
+									1]);
 
 		if (!levdatum)
 			return -EINVAL;
@@ -551,11 +583,16 @@ int mls_convert_context(policydb_t * oldp,
 			if (ebitmap_node_get_bit(cnode, i)) {
 				int rc;
 
-				catdatum = (cat_datum_t *) hashtab_search(newp->p_cats.table,
-					 oldp->p_cat_val_to_name[i]);
+				catdatum =
+				    (cat_datum_t *) hashtab_search(newp->p_cats.
+								   table,
+								   oldp->
+								   p_cat_val_to_name
+								   [i]);
 				if (!catdatum)
 					return -EINVAL;
-				rc = ebitmap_set_bit(&bitmap, catdatum->value - 1, 1);
+				rc = ebitmap_set_bit(&bitmap,
+						     catdatum->s.value - 1, 1);
 				if (rc)
 					return rc;
 			}
@@ -567,30 +604,26 @@ int mls_convert_context(policydb_t * oldp,
 	return 0;
 }
 
-int mls_compute_sid(policydb_t *policydb,
-		    context_struct_t *scontext,
-		    context_struct_t *tcontext,
+int mls_compute_sid(policydb_t * policydb,
+		    context_struct_t * scontext,
+		    context_struct_t * tcontext,
 		    sepol_security_class_t tclass,
-		    uint32_t specified,
-		    context_struct_t *newcontext)
+		    uint32_t specified, context_struct_t * newcontext)
 {
+	range_trans_t *rtr;
 	if (!policydb->mls)
 		return 0;
 
 	switch (specified) {
 	case AVTAB_TRANSITION:
-		if (tclass == SECCLASS_PROCESS) {
-			range_trans_t *rangetr;
-
-			/* Look for a range transition rule. */
-			for (rangetr = policydb->range_tr; rangetr;
-			     rangetr = rangetr->next) {
-				if (rangetr->dom == scontext->type &&
-				    rangetr->type == tcontext->type) {
-					/* Set the range from the rule */
-					return mls_range_set(newcontext,
-					                     &rangetr->range);
-				}
+		/* Look for a range transition rule. */
+		for (rtr = policydb->range_tr; rtr; rtr = rtr->next) {
+			if (rtr->source_type == scontext->type &&
+			    rtr->target_type == tcontext->type &&
+			    rtr->target_class == tclass) {
+				/* Set the range from the rule */
+				return mls_range_set(newcontext,
+						     &rtr->target_range);
 			}
 		}
 		/* Fallthrough */
@@ -617,17 +650,15 @@ int mls_compute_sid(policydb_t *policydb,
 	return -EINVAL;
 }
 
-int sepol_mls_contains(
-	sepol_handle_t* handle,
-	sepol_policydb_t* policydb,
-	const char* mls1,
-	const char* mls2,
-	int* response) {
+int sepol_mls_contains(sepol_handle_t * handle,
+		       sepol_policydb_t * policydb,
+		       const char *mls1, const char *mls2, int *response)
+{
 
 	context_struct_t *ctx1 = NULL, *ctx2 = NULL;
 	ctx1 = malloc(sizeof(context_struct_t));
 	ctx2 = malloc(sizeof(context_struct_t));
-	if (ctx1 == NULL || ctx2 == NULL) 
+	if (ctx1 == NULL || ctx2 == NULL)
 		goto omem;
 	context_init(ctx1);
 	context_init(ctx2);
@@ -645,31 +676,30 @@ int sepol_mls_contains(
 	free(ctx2);
 	return STATUS_SUCCESS;
 
-	omem:
+      omem:
 	ERR(handle, "out of memory");
 
-	err:
-	ERR(handle, "could not check if mls context %s contains %s", 
-		mls1, mls2);
+      err:
+	ERR(handle, "could not check if mls context %s contains %s",
+	    mls1, mls2);
 	context_destroy(ctx1);
 	context_destroy(ctx2);
 	free(ctx1);
 	free(ctx2);
-	return STATUS_ERR;	
+	return STATUS_ERR;
 }
 
-int sepol_mls_check(
-	sepol_handle_t* handle,
-	sepol_policydb_t* policydb,
-	const char* mls) {
+int sepol_mls_check(sepol_handle_t * handle,
+		    sepol_policydb_t * policydb, const char *mls)
+{
 
 	int ret;
-	context_struct_t* con = malloc(sizeof(context_struct_t));
+	context_struct_t *con = malloc(sizeof(context_struct_t));
 	if (!con) {
 		ERR(handle, "out of memory, could not check if "
-			"mls context %s is valid", mls);
+		    "mls context %s is valid", mls);
 		return STATUS_ERR;
-	} 
+	}
 	context_init(con);
 
 	ret = mls_from_string(handle, &policydb->p, mls, con);
@@ -677,3 +707,94 @@ int sepol_mls_check(
 	free(con);
 	return ret;
 }
+
+void mls_semantic_cat_init(mls_semantic_cat_t * c)
+{
+	memset(c, 0, sizeof(mls_semantic_cat_t));
+}
+
+void mls_semantic_cat_destroy(mls_semantic_cat_t * c __attribute__ ((unused)))
+{
+	/* it's currently a simple struct - really nothing to destroy */
+	return;
+}
+
+void mls_semantic_level_init(mls_semantic_level_t * l)
+{
+	memset(l, 0, sizeof(mls_semantic_level_t));
+}
+
+void mls_semantic_level_destroy(mls_semantic_level_t * l)
+{
+	mls_semantic_cat_t *cur, *next;
+
+	if (l == NULL)
+		return;
+
+	next = l->cat;
+	while (next) {
+		cur = next;
+		next = cur->next;
+		mls_semantic_cat_destroy(cur);
+		free(cur);
+	}
+}
+
+int mls_semantic_level_cpy(mls_semantic_level_t * dst,
+			   mls_semantic_level_t * src)
+{
+	mls_semantic_cat_t *cat, *newcat, *lnewcat = NULL;
+
+	mls_semantic_level_init(dst);
+	dst->sens = src->sens;
+	cat = src->cat;
+	while (cat) {
+		newcat =
+		    (mls_semantic_cat_t *) malloc(sizeof(mls_semantic_cat_t));
+		if (!newcat)
+			goto err;
+
+		mls_semantic_cat_init(newcat);
+		if (lnewcat)
+			lnewcat->next = newcat;
+		else
+			dst->cat = newcat;
+
+		newcat->low = cat->low;
+		newcat->high = cat->high;
+
+		lnewcat = newcat;
+		cat = cat->next;
+	}
+	return 0;
+
+      err:
+	mls_semantic_level_destroy(dst);
+	return -1;
+}
+
+void mls_semantic_range_init(mls_semantic_range_t * r)
+{
+	mls_semantic_level_init(&r->level[0]);
+	mls_semantic_level_init(&r->level[1]);
+}
+
+void mls_semantic_range_destroy(mls_semantic_range_t * r)
+{
+	mls_semantic_level_destroy(&r->level[0]);
+	mls_semantic_level_destroy(&r->level[1]);
+}
+
+int mls_semantic_range_cpy(mls_semantic_range_t * dst,
+			   mls_semantic_range_t * src)
+{
+	if (mls_semantic_level_cpy(&dst->level[0], &src->level[0]) < 0)
+		return -1;
+
+	if (mls_semantic_level_cpy(&dst->level[1], &src->level[1]) < 0) {
+		mls_semantic_level_destroy(&dst->level[0]);
+		return -1;
+	}
+
+	return 0;
+}
